@@ -1,10 +1,12 @@
 from itertools import chain
+from shutil import which
 import os
 import platform
 import subprocess
 import sys
 import sysconfig
 
+from filesystems.click import PATH
 import attr
 import click
 import filesystems.native
@@ -14,7 +16,7 @@ def _create_virtualenv(virtualenv, arguments, python, stdout, stderr):
     subprocess.check_call(
         [
             sys.executable, "-m", "virtualenv",
-            "--python", python,
+            "--python", which(python),
             "--quiet",
         ] + list(arguments) + [str(virtualenv.path)],
         stderr=stderr,
@@ -136,7 +138,7 @@ class _Locator(click.ParamType):
     def convert(self, value, param, context):
         if not isinstance(value, str):
             return value
-        return Locator(root=filesystems.Path.from_string(str(value)))
+        return Locator(root=PATH.convert(value, param, context))
 
 
 _ROOT = click.option(
@@ -156,6 +158,7 @@ _LINK_DIR = click.option(
     default=filesystems.Path.from_string(
         sysconfig.get_path("scripts", "posix_user"),
     ),
+    type=PATH,
     help="The directory to link scripts into.",
 )
 
