@@ -1,3 +1,5 @@
+from datetime import datetime, timedelta
+from tempfile import NamedTemporaryFile
 from unittest import TestCase
 import os
 
@@ -111,6 +113,72 @@ class TestConverge(CLIMixin, TestCase):
             ),
         )
 
+    def test_it_runs_post_commands(self):
+        self.assertFalse(self.locator.for_name("a").exists_on(self.filesystem))
+
+        file = NamedTemporaryFile(delete=False)
+        self.addCleanup(os.remove, file.name)
+        mtime = os.path.getmtime(file.name)
+        new_mtime = datetime.fromtimestamp(mtime) + timedelta(minutes=10)
+
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), f"""
+            [virtualenv.a]
+            post-commands = [
+                ["true"],
+                ["touch", "-t", "{new_mtime:%Y%m%d%H%M}", "{file.name}"],
+            ]
+            """
+        )
+
+        self.run_cli(["converge"])
+
+        self.assertTrue(self.locator.for_name("a").exists_on(self.filesystem))
+        self.assertGreater(os.path.getmtime(file.name), mtime)
+
+    def test_it_does_not_run_post_commands_for_already_converged_envs(self):
+        self.assertFalse(self.locator.for_name("a").exists_on(self.filesystem))
+
+        file = NamedTemporaryFile(delete=False)
+        self.addCleanup(os.remove, file.name)
+
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), f"""
+            [virtualenv.a]
+            post-commands = [
+                ["true"],
+                ["touch", "{file.name}"],
+            ]
+            """
+        )
+
+        self.run_cli(["converge"])
+        mtime = os.path.getmtime(file.name)
+
+        self.run_cli(["converge"])
+        self.assertEqual(os.path.getmtime(file.name), mtime)
+
+    def test_it_stops_post_commands_on_error(self):
+        self.assertFalse(self.locator.for_name("a").exists_on(self.filesystem))
+
+        file = NamedTemporaryFile(delete=False)
+        self.addCleanup(os.remove, file.name)
+        mtime = os.path.getmtime(file.name)
+
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), f"""
+            [virtualenv.a]
+            post-commands = [
+                ["false"],
+                ["touch", "{file.name}"],
+            ]
+            """
+        )
+
+        self.run_cli(["converge"])
+
+        self.assertEqual(os.path.getmtime(file.name), mtime)
+
     def test_bundles(self):
         self.assertFalse(self.locator.for_name("a").exists_on(self.filesystem))
 
@@ -189,7 +257,7 @@ class TestConverge(CLIMixin, TestCase):
             self.locator.root.descendant("virtualenvs.toml"), """
             [virtualenv.a]
             [virtualenv.b]
-            [virtualenv.magicExplodingVirtualenv]
+            [virtualenv.magicExplodingVirtualenvOnInstall]
             [virtualenv.c]
             """
         )
@@ -205,13 +273,65 @@ class TestConverge(CLIMixin, TestCase):
             ),
             tuple((set(), set()) for _ in "abc") + (True,),
         )
+        self.assertIn(
+            "'magicExplodingVirtualenvOnInstall' failed",
+            self.stderr.getvalue(),
+        )
 
     def test_it_can_be_asked_to_blow_up_immediately_on_install(self):
         self.filesystem.set_contents(
             self.locator.root.descendant("virtualenvs.toml"), """
             [virtualenv.a]
             [virtualenv.b]
-            [virtualenv.magicExplodingVirtualenv]
+            [virtualenv.magicExplodingVirtualenvOnInstall]
+            [virtualenv.c]
+            """
+        )
+
+        with self.assertRaises(ZeroDivisionError):
+            self.run_cli(["converge", "--fail-fast"])
+
+        self.assertEqual(
+            (
+                self.installed(self.locator.for_name("a")),
+                self.installed(self.locator.for_name("b")),
+                self.locator.for_name("c").exists_on(self.filesystem),
+            ),
+            ((set(), set()), (set(), set()), False),
+        )
+
+    def test_it_does_not_blow_up_by_default_on_create(self):
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), """
+            [virtualenv.a]
+            [virtualenv.b]
+            [virtualenv.magicExplodingVirtualenvOnCreate]
+            [virtualenv.c]
+            """
+        )
+
+        self.run_cli(["converge"])
+
+        self.assertEqual(
+            (
+                self.installed(self.locator.for_name("a")),
+                self.installed(self.locator.for_name("b")),
+                self.installed(self.locator.for_name("c")),
+                self.locator.for_name("c").exists_on(self.filesystem),
+            ),
+            tuple((set(), set()) for _ in "abc") + (True,),
+        )
+        self.assertIn(
+            "'magicExplodingVirtualenvOnCreate' failed",
+            self.stderr.getvalue(),
+        )
+
+    def test_it_can_be_asked_to_blow_up_immediately_on_create(self):
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), """
+            [virtualenv.a]
+            [virtualenv.b]
+            [virtualenv.magicExplodingVirtualenvOnCreate]
             [virtualenv.c]
             """
         )
@@ -244,6 +364,22 @@ class TestConverge(CLIMixin, TestCase):
             (set(), set()),
         )
 
+    def test_custom_link_dir(self):
+        self.filesystem.set_contents(
+            self.locator.root.descendant("virtualenvs.toml"), """
+            [virtualenv.a]
+            link = ["foo"]
+            """
+        )
+
+        link_dir = self.link_dir.descendant("some", "child", "bin")
+        self.run_cli(["converge", "--link-dir", str(link_dir)])
+
+        self.assertEqual(
+            self.filesystem.readlink(link_dir / "foo"),
+            self.locator.for_name("a").binary("foo"),
+        )
+
     def test_link_exists(self):
         self.filesystem.set_contents(
             self.locator.root.descendant("virtualenvs.toml"), """
