@@ -7,7 +7,7 @@ import click.testing
 import filesystems.memory
 
 from venvs import _cli, _config
-from venvs.common import Locator, VirtualEnv, _EX_OK
+from venvs.common import _EX_OK, Locator, VirtualEnv
 
 
 class CLIMixin(object):
@@ -67,6 +67,10 @@ class CLIMixin(object):
         return packages, reqs
 
     def _fake_create(self, virtualenv, **kwargs):
+        # FIXME: ...
+        if virtualenv.path.basename() == "magicexplodingvirtualenvoncreate":
+            raise ZeroDivisionError("Hey you told me to blow up on create!")
+
         try:
             self.filesystem.create_directory(path=virtualenv.path.parent())
         except FileExists:
@@ -79,8 +83,8 @@ class CLIMixin(object):
 
     def _fake_install(self, virtualenv, packages, requirements, **kwargs):
         # FIXME: ...
-        if virtualenv.path.basename() == "magicexplodingvirtualenv":
-            raise ZeroDivisionError("Hey you told me to blow up!")
+        if virtualenv.path.basename() == "magicexplodingvirtualenvoninstall":
+            raise ZeroDivisionError("Hey you told me to blow up on install!")
 
         base = virtualenv.path
         with self.filesystem.open(base / "packages", "at") as f:
@@ -91,7 +95,7 @@ class CLIMixin(object):
     def run_cli(self, argv=(), exit_status=_EX_OK):
         runner = click.testing.CliRunner()
         default_map = dict(
-            link_dir=self.link_dir,
+            link_dir=str(self.link_dir),
             locator=self.locator,
             filesystem=self.filesystem,
         )
